//  gcc main.c -g -lX11 -lXext -lm
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>

Display *display;
Window window;
XIC xic;
Pixmap CreatePixmapFromRAW(Display *display, Window window, unsigned char *image, int width, int height);

#include "skin.h"

/* Set no frame WM hint. From SDL sources. */
void SetNoBorder(Display *dpy, Window win, Window root){
	int set = 0;
	Atom WM_HINTS;
	/* First try to set MWM hints */
	WM_HINTS = XInternAtom(dpy, "_MOTIF_WM_HINTS", True);
	/* Hints used by Motif compliant window managers */
	if(WM_HINTS != None){
		struct{
			unsigned long flags;
			unsigned long functions;
			unsigned long decorations;
			long input_mode;
			unsigned long status;
		} MWMHints = { (1L << 1), 0, 0, 0, 0 };

		XChangeProperty(dpy, win, WM_HINTS, WM_HINTS, 32,
		PropModeReplace, (unsigned char *)&MWMHints,
		sizeof(MWMHints)/sizeof(long));
		set = 1;
	}

	/* Now try to set KWM hints */
	WM_HINTS = XInternAtom(dpy, "KWM_WIN_DECORATION", True);
	if(WM_HINTS != None){
		long KWMHints = 0;
		XChangeProperty(dpy, win, WM_HINTS, WM_HINTS, 32,
		PropModeReplace, (unsigned char *)&KWMHints,
		sizeof(KWMHints)/sizeof(long));
		set = 1;
	}

	/* Now try to set GNOME hints */
	WM_HINTS = XInternAtom(dpy, "_WIN_HINTS", True);
	if(WM_HINTS != None){
		long GNOMEHints = 0;
		XChangeProperty(dpy, win, WM_HINTS, WM_HINTS, 32,
		PropModeReplace, (unsigned char *)&GNOMEHints,
		sizeof(GNOMEHints)/sizeof(long));
		set = 1;
	}

	/* Finally set the transient hints if necessary */
	if(!set)XSetTransientForHint(dpy, win, root);
}

//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
// Convert from RAW24
Pixmap CreatePixmapFromRAW(Display *display, Window window, unsigned char *image, int width, int height){
	XImage *img;
	Pixmap pix;
	unsigned long idx;
	char *image32=(char *)malloc(width*height*4);
	unsigned long tmp;
	for(idx = 0; idx < width * height; idx++){
		tmp = *(long *)(image + idx * 3);
		tmp &= 0xffffff;
		*(long *)(image32 + idx * 4) = tmp;
	}
	
	img = XCreateImage(display, DefaultVisual(display, 0), 24, ZPixmap, 0, image32, width, height, 32, 0);
	pix = XCreatePixmap(display, window, width, height, DefaultDepth(display, 0));
	XPutImage(display, pix, DefaultGC(display, 0), img, 0, 0, 0, 0, width, height);
	XDestroyImage(img);
	return pix;
}

//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

void processEvent(Display *display, Window window, Pixmap pix, int width, int height){
	XEvent ev;
	XNextEvent(display, &ev);
	XWindowAttributes wa;
	int ofs;
	unsigned long kcode;

	static int x, y;
	switch(ev.type){
	case Expose:
		while(XCheckTypedWindowEvent(display, window, Expose, &ev));

		// XCopyArea(display, src, dest, gc, src_x, src_y, width, height,  dest_x, dest_y)
		XCopyArea(display, pix, window, DefaultGC(display, 0), 0, 0, width, height,  0, 0);

		// Create title bar
		XGetWindowAttributes(display, window, &wa);
		XCopyArea(display, left_a.pmap, window, DefaultGC(display, 0), 0, 0, left_a.width, left_a.height,  0, 0);
		for(ofs = left_a.width; ofs < wa.width - oper_a.width; ofs += base_a.width){
			XCopyArea(display, base_a.pmap, window, DefaultGC(display, 0), 0, 0, base_a.width, base_a.height, ofs, 0);
		}
		XCopyArea(display, oper_a.pmap, window, DefaultGC(display, 0), 0, 0, oper_a.width, oper_a.height,  wa.width - oper_a.width, 0);

		break;
		
	case KeyPress:
		kcode = 0;
		Xutf8LookupString(xic, &ev.xkey, (char*)&kcode, sizeof(kcode), NULL, NULL);
		fprintf(stderr, "state: 0x%08x keycode: 0x%02x -> %s\n", ev.xkey.state, ev.xkey.keycode, &kcode);
		break;
		
	case ButtonPress:
		switch(ev.xbutton.button){
		case Button1:
			x = ev.xbutton.x;
			y = ev.xbutton.y;
			break;
			
		case Button3:
			exit(0);
		}

	case MotionNotify:
		if(ev.xmotion.state & Button1Mask){
			//XDrawPoint(display, window, DefaultGC(display, 0), ev.xmotion.x, ev.xmotion.y);
			XGetWindowAttributes(display, window, &wa);
			XMoveWindow(display, window, wa.x + ev.xmotion.x - x, wa.y + ev.xmotion.y - y);
		}
	}
}

//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

int main(int argc, char **argv){
	Pixmap pix;
	int width = 512, height = 512;
	display = XOpenDisplay(NULL);

	if(DefaultDepth(display, 0) < 24){
		fprintf(stderr, "Cannot work in non true color mode\n");
		exit(1);
	}

	XIM xim = XOpenIM(display, NULL, NULL, NULL);
	xic = XCreateIC(xim, XNInputStyle, XIMPreeditNothing | XIMStatusNothing, 0);
	
	Visual *visual = DefaultVisual(display, 0);
	window = XCreateSimpleWindow(display, RootWindow(display, 0), 0, 0, 1 /*width*/, 1 /*height*/, 1, 0, 0);
	SetNoBorder(display, window, RootWindow(display, 0));

	// Create sample raw image 24bpp
	int i, j;
	unsigned char *raw_image=(unsigned char *)malloc(width*height*3);
	unsigned char *p = raw_image;
	
	for(i=0; i<width; i++){
		for(j = 0; j < height; j++){
			if((i < 256) && (j < 256)){
				*p++=rand()%256; // blue
				*p++=rand()%256; // green
				*p++=rand()%256; // red
			}else{
				*p++=i%256; // blue
				*p++=j%256; // green
				if(i<256)*p++=i%256; // red
				else if(j<256)*p++=j%256; // red
					else *p++=(256-j)%256; // red
			}
		}
	}

	pix = CreatePixmapFromRAW(display, window, raw_image, width, height);

	XSelectInput(display, window, ButtonPressMask|ExposureMask|Button1MotionMask|KeyPressMask);
	XMapWindow(display, window);
	//XMoveWindow(display, window, 100, 100);
	XMoveResizeWindow(display, window, 100, 100, width, height);

	read_skin();

	while(1)processEvent(display, window, pix, width, height);
}
